import 'package:flutter/material.dart';
import 'package:global/global.dart';

abstract class GWebViewRoute extends StatefulWidget {
  final Map<String, dynamic> arguments;

  const GWebViewRoute(this.arguments, {super.key});
}

abstract class GWebViewRouteState<T extends GWebViewRoute>
    extends State<GWebViewRoute> {
  late String pageTitle;
  late InAppWebViewController webviewController;
  late InAppWebView webView;
  Uri? initialUri;

  @override
  void initState() {
    super.initState();
    initialUri = Uri.tryParse(widget.arguments[GKeys.initialWebviewData]);
    pageTitle = GTexts.untitled;
    webView = InAppWebView(
      initialData: initialUri == null
          ? InAppWebViewInitialData(
              data: widget.arguments[GKeys.initialWebviewData])
          : null,
      initialUrlRequest:
          initialUri == null ? null : URLRequest(url: WebUri.uri(initialUri!)),
      initialSettings: InAppWebViewSettings(
        useShouldOverrideUrlLoading: true,
        useOnDownloadStart: true,
        useHybridComposition: true,
        allowsInlineMediaPlayback: true,
      ),
      onWebViewCreated: (controller) {
        webviewController = controller;
        widget.arguments[GKeys.webviewController] = webviewController;
      },
      shouldOverrideUrlLoading: (controller, navigationAction) async {
        var uri = navigationAction.request.url!;
        if (!GUtils.whiteListedSchemesForWV.contains(uri.scheme)) {
          if (await canLaunchUrl(uri)) {
            await launchUrl(uri);
            return NavigationActionPolicy.CANCEL;
          }
        }
        return NavigationActionPolicy.ALLOW;
      },
      onTitleChanged: (controller, title) {
        if (title != null && title.isNotEmpty) {
          setState(() => pageTitle = title);
        }
      },
      onDownloadStartRequest: (_, downloadStartReq) =>
          GUtils.onFileDownloadStartRequest(url: downloadStartReq.url.rawValue),
    );
  }
}
